class TrackableObject():

    def __init__(self,name):
        self._name=name
    
    @property
    def name(self):
        return self._name
    
    @name.setter
    def name(self, new_name):
        if isinstance(new_name,str):
            self._name = new_name

    def track_global_coordinates(self):
        """
        Method used to track objects
        Retruns:
            latitude, longitude - tuple of ints
        """
        pass

# t = TrackableObj()
# t.name = "poksdasdpok"
# print(t.name)


"""
CONSTANTS 
"""
import enum
from pathlib import Path

VERSION = "1.1.0"
#ms between calling while_waiting on the forms 
KEYPRESS_TIMEOUT =  100
#Default Offset Delta
DEFAULT_OFFSET_DELTA = 20


ROOT_DIR = Path(__file__).parent
DATA_DIR = (ROOT_DIR/"data").resolve()
MAP_IMAGE_FILEPATH = (  DATA_DIR / "mapimg.png").resolve()
MAP_IMAGE_THRESHOLD = 225

class API_TYPES(enum.Enum):
    LAUNCHES = "UPCOMING LAUNCHES"
    EVENTS = "UPCOMING EVENTS"
    ASTRONAUTS = "ACTIVE ASTRONAUTS"
    SPACE_STATIONS = "ACTIVE SPACE STATIONS"
    SOLAR_SYSTEM_BODIES = "SOLAR SYSTEM BODIES"

MSG_CONNECTION_ERROR = "Could not connect to the Internet\nTrying again in {} seconds".format(KEYPRESS_TIMEOUT/10)

CONTROLS = {
    "update" : "^R",
    "track" : "^T",
    "increment_offset":">",
    "decrement_offset":"<",
}

MSG_CONTROLS_HELP = """
Controls: 
    Access the Main Menu with 'CONTROL+X'
    Use the ARROW KEYS to navigate the lists, press ENTER to select an option
    Flip through the pages with '<' ( SHIFT and , ) and '>' ( SHIFT and . )
    Track a selected object with 'CONTROL+T'
"""



import setuptools
import worden.const
with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="worden",
    version=worden.const.VERSION,
    author="João Guimarães",
    author_email="jvlsg-github@ipriva.org",
    description="TUI Space operations center",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jvlsg/Worden",
    entry_points={'console_scripts': ['worden=worden.__main__:run_app'], },
    packages=setuptools.find_packages() 
    + ["worden.src","worden.src.api","worden.src.ui"],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Environment :: Console :: Curses",
        "Intended Audience :: Education",
        "Natural Language :: English"
    ],
    python_requires='>=3.7',
    install_requires=[
        "Pillow>=5.4.1",
        "drawille>=0.1.0",
        "npyscreen>=4.10.5"],    
)


import npyscreen, curses
import logging
import worden.const as const
from PIL import Image

class TextBox(npyscreen.BoxTitle):
    # MultiLineEdit now will be surrounded by boxing
    _contained_widget = npyscreen.MultiLineEdit

class WordenForm(npyscreen.FormBaseNewWithMenus):
    """
    Generic npyscreen Form used by other Forms containing Standard Handlers
    """

    def create(self, *args, **keywords):
        super(WordenForm, self).create(*args, **keywords)
        
        self.USEABLE_Y, self.USEABLE_X = self.useable_space()
        self.PADDING_X = 1
        self.PADDING_Y = 1
        #None or one of the values described in API_TYPES Enum
        self.api_type = None
        self.menu = self.add_menu(name="Main Menu")
        self.menu.addItem(text="MAP",onSelect=self.h_change_form,arguments=["MAIN"])
        for api_type in const.API_TYPES:
            self.menu.addItem(text=api_type.value,onSelect=self.h_change_form,arguments=[api_type])
        self.menu.addItem(text="EXIT",onSelect=self.h_close_application)

        new_handlers={
            const.CONTROLS["update"] : self.h_update,
            const.CONTROLS["track"] : self.h_track_object,
            const.CONTROLS["increment_offset"] : self.h_increment_offset,
            const.CONTROLS["decrement_offset"] : self.h_decrement_offset,
            }
        self.add_handlers(new_handlers)
    
    def h_close_application(self):
        self.parentApp.setNextForm(None)
        self.editing = False
        logging.debug("Closing Application")
        self.parentApp.switchFormNow()

    def h_change_form(self,*args,**keywords):
        """
        Changes the current active form of the App
        """
        if args[0] not in list(self.parentApp._Forms.keys()):
            err_str = "Invalid Form name: {}".format(args[0])
            logging.error(err_str)
            raise KeyError(err_str)
        logging.debug("Changing Form to {}".format(args[0]))
        self.parentApp.change_form_to(args[0])

    
    def h_increment_offset(self, *args, **keywords):
        """
        Method used by forms that manipulate API data, to fetch the next values
        """
        pass

    def h_decrement_offset(self, *args, **keywords):
        """
        Method used by forms that manipulate API data, to fetch the previous values
        """
        pass

    def h_track_object(self, *args,**keywords):
        """
        Wrapper Used by Keyboard Handler to invoke the Form's Track function
        """
        self.track_object()

    def track_object(self):
        """
        Function that Track a currently selected.
        """
        pass

    def h_update(self,_input):
        """
        Wrapper Used by Keyboard Handler to invoke the Form's Update function
        """
        self.update_form()

    def update_form(self):
        """
        Function that updates form.
        If necessary, it should contain the functions for a basic Update/Draw loop
        """
        pass

def draw_image_on_canvas(image_descriptor=None,color_threshold=0,invert_threshold=False,canvas=None,pixel_limits=None):
    """
    Draws an image onto a Canvas

    Args:
        image_descriptor: open in binary mode e.g. open(MAP_IMAGE_FILEPATH,'rb') requests.get('...',stream=True)
        color_threshold: int (0-255) to paint (or not) a pixel in drawille
        invert_threshold: if set to True, the threshold test will be: paint when pixel color > threshold 
        canvas: The canvas to draw
        pixel_limits: a dict of X,Y Coordinates that limit the size of the image, used by canvas.frame(...)
            If the image size is greater than the limits, the image is resized maintaining the ratio and
            the pixel limits are updated 
            pixel_limits = {"min_x":0,"min_y":0,"max_x":128,"max_y":128}

    Returns: None. 
    """

    #Determine if the size of the image is greater than the size to print, if so, scale it down
    i = Image.open(image_descriptor).convert(mode='L')
    image_width, image_height = i.size   

    w_ratio = 1    
    if  pixel_limits["max_x"] < image_width:
        w_ratio =  pixel_limits["max_x"] / float(image_width)

    h_ratio = 1
    if pixel_limits["max_y"] < image_width:
        h_ratio = pixel_limits["max_y"] / float(image_height)
    
    ratio = 0.8*min([w_ratio,h_ratio])
    image_width = int(image_width * ratio)
    image_height = int(image_height * ratio)
    i = i.resize((image_width, image_height), Image.ANTIALIAS)
    #Update limits
    pixel_limits["max_y"] = image_height
    pixel_limits["max_x"] = image_width
            
    try:
        i_converted = i.tobytes()
    except AttributeError:
        raise
    
    if invert_threshold:
        threshold_test = lambda pix_color,color_threshold: pix_color > color_threshold
    else:
        threshold_test = lambda pix_color,color_threshold: pix_color < color_threshold

    x = y = 0
    for pix in i_converted:
        if threshold_test(pix,color_threshold):
            canvas.set(x, y)
        x += 1
        if x >= image_width:
            y += 1
            x = 0


#!/usr/bin/env python
import curses
import logging

import npyscreen

from worden.src.api import api_man
from worden.src.api.trackable_object import TrackableObject
from worden.src.ui.list_and_details_form import ListAndDetailsForm
from worden.src.ui.map_form import MapForm
import worden.const as const

class WordenApp(npyscreen.NPSAppManaged):
    """
    The core of the application's Logic will be here.
    The Form classes should only know what is required to display themselves
    """

    def while_waiting(self):
        #UPDATE THE CURRENT FORM
        self._Forms[self._active_form].update_form()

        if self.tracked_object != None and self._active_form != self.tracked_object_type:
            #Invokes the while_waiting , which will in turn update the list of objects
            self._Forms[self.tracked_object_type].while_waiting()
            #Gets the equivalent to the tracked object again
            self.tracked_object = self.api_man.pages.get(self.tracked_object_type).results_dict.get(self.tracked_object_key)
    
    def onStart(self):
        #THIS NEEDS TO BE BEFORE REGISTERING THE FORM 
        self.keypress_timeout_default = const.KEYPRESS_TIMEOUT
        self.api_man = api_man.Api_Manager(self)
        self.f_map = MapForm(parentApp=self, name="MAPS")
        self.registerForm("MAIN",self.f_map)

        self.f_api_forms = {}
        for api_type in const.API_TYPES:
            f_api_type =  ListAndDetailsForm(parentApp=self,name=api_type.value)
            self.f_api_forms[api_type] = f_api_type
            f_api_type.set_api_type(api_type)
            self.registerForm(api_type,self.f_api_forms[api_type])

        self.f_api_forms[const.API_TYPES.SOLAR_SYSTEM_BODIES].set_refresh_api_data(False)

        self.f_api_forms[const.API_TYPES.ASTRONAUTS].set_order_keys(True)
        self.f_api_forms[const.API_TYPES.SOLAR_SYSTEM_BODIES].set_order_keys(True)

        self._active_form = "MAIN"
        self.tracked_object = None
        self.tracked_object_key = None
        self.tracked_object_type = None


    def set_tracked_object(self,trackable_object=None,trackable_object_key=None,trackable_object_type=None):
        """
        Sets the new tracked object, it's key and API Type
        """

        if not issubclass(type(trackable_object),TrackableObject) or trackable_object_key == None or trackable_object_type == None:
            raise TypeError()
        logging.debug("Set Tracked Object to a {}: {}".format(type(trackable_object),trackable_object_key))
        self.tracked_object = trackable_object
        self.tracked_object_key = trackable_object_key
        self.tracked_object_type = trackable_object_type

    def onCleanExit(self):
        npyscreen.notify_wait("Goodbye!")

    def change_form_to(self, form_name):
        """
        Changes the App's current active form 
        """

        if form_name not in self._Forms.keys():
            return
        self._active_form = form_name
        self.switchForm(form_name)
        self._Forms[self._active_form].update_form()
        self.resetHistory()
